//+------------------------------------------------------------------+
//|                                                  MarketDepth.mq4 |
//|                                         Copyright 2014, TheXpert |
//|                                          http://www.trend-lab.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright 2014, TheXpert"
#property link      "http://www.trend-lab.ru/"
#property strict

enum Exchanges
{
   ExchangeBtce, //Btc-e exchange
   ExchangeBitfinex, //Bitfinex exchange
   ExchangeBitstamp, //Bitstamp exchange
   ExchangeHuobi, //Huobi exchange
   ExchangeOkcoin, //OkCoin exchange
   ExchangeCoinBase, //CoinBase exchange
   ExchangePoloniex, //Poloniex exchange
};

#property show_inputs

input int Level2Depth = 100;
input int RefreshSeconds = 2;
input bool AutoPointSize = true;
input double PointSize = 0.2;
input Exchanges Exchange = ExchangeBtce;
input string CustomInstrument = "";
input bool AutoScale = false;
input double CustomScale = 1.0;

int ScaleDigits = 0;

input color Back = Black;

input color Bulls = DarkGreen;
input color BullLevels = LightGreen;
input color InnerBullLevels = SeaGreen;

input color Bears = 0x64;
input color BearLevels = Salmon;
input color InnerBearLevels = Brown;

input color ScaleColor = Yellow;
input color CumScaleColor = LightBlue;
input color PriceColor = LightGray;

class IExchange
{
public:
   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      Alert("Implementation failure!");
      return NULL;
   }
   
   virtual string RequestUrl() const
   {
      Alert("Implementation failure!");
      return NULL;
   }

   virtual string AdaptSymbol(string s) const
   {
      Alert("Implementation failure!");
      return NULL;
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      Alert("Implementation failure!");
      return false;
   }
};

class BtceExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      string lower = ToLower(s);
      return (StringConcatenate(StringSubstr(lower, 0, 3), "_", StringSubstr(lower, 3, 3)));
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = StringFormat("https://btc-e.com/api/2/%s/depth", symbol);
      if (depthLimit != -1) s = StringConcatenate(s, "/" + IntegerToString(depthLimit));
      return s;
   }
   
   virtual string RequestUrl() const
   {
      return "https://btc-e.com/api";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int askStart = StringFind(s, "[[");
      int askEnd = StringFind(s, "]]", askStart);
      int bidStart = StringFind(s, "[[", askEnd);
      int bidEnd = StringFind(s, "]]", bidStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }
   
private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = StringSubstr(pair, 0, pos);
      string vStr = StringSubstr(pair, pos + 1);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class BitfinexExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      return ToLower(s);
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = StringFormat("https://api.bitfinex.com/v1/book/%s", symbol);
      if (depthLimit != -1) s = StringConcatenate(s, StringFormat("?limit_asks=%d&limit_bids=%d", depthLimit, depthLimit));
      return s;
   }
   
   virtual string RequestUrl() const
   {
      return "https://api.bitfinex.com/v1";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int bidStart = StringFind(s, "[{");
      int bidEnd = StringFind(s, "}]", bidStart);
      int askStart = StringFind(s, "[{", bidEnd);
      int askEnd = StringFind(s, "}]", askStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "},{");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "},{");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }
      
      return (true);
   }   

private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, "\"price\":\"");
      if (pos == -1)
      {
         return (false);
      }
      int endPos = StringFind(pair, "\"", pos + 9);
      if (endPos == -1)
      {
         return (false);
      }
      string pStr = StringSubstr(pair, pos + 9, endPos - pos - 9);


      pos = StringFind(pair, "\"amount\":\"");
      if (pos == -1)
      {
         return (false);
      }
      endPos = StringFind(pair, "\"", pos + 10);
      if (endPos == -1)
      {
         return (false);
      }
      string vStr = StringSubstr(pair, pos + 10, endPos - pos - 10);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class BitstampExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      return s;
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      return "https://www.bitstamp.net/api/order_book/";
   }
   
   virtual string RequestUrl() const
   {
      return "https://www.bitstamp.net/api/";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      StringReplace(s, " ", "");
      
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int bidStart = StringFind(s, "[[");
      int bidEnd = StringFind(s, "]]", bidStart);
      int askStart = StringFind(s, "[[", bidEnd);
      int askEnd = StringFind(s, "]]", askStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }   

private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = StringSubstr(pair, 1, pos - 2);
      string vStr = StringSubstr(pair, pos + 2, StringLen(pair) - pos - 3);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class HuobiExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      return s;
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = ToLower(symbol);
      if (s == "ltcusd" || s == "ltccnh" || s == "ltccny")
      {
         return "http://market.huobi.com/staticmarket/depth_ltc_json.js";
      }

      return "http://market.huobi.com/staticmarket/depth_btc_json.js";;
   }
   
   virtual string RequestUrl() const
   {
      return "http://market.huobi.com/";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int askStart = StringFind(s, "[[");
      int askEnd = StringFind(s, "]]", askStart);
      int bidStart = StringFind(s, "[[", askEnd);
      int bidEnd = StringFind(s, "]]", bidStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }
   
private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = StringSubstr(pair, 0, pos);
      if (StringGetChar(pStr, 0) == '\"') pStr = StringSubstr(pStr, 1, StringLen(pStr) - 2);
      string vStr = StringSubstr(pair, pos + 1);
      if (StringGetChar(vStr, 0) == '\"') vStr = StringSubstr(vStr, 1, StringLen(vStr) - 2);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class OkcoinExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      return s;
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = ToLower(symbol);
      if (s == "ltcusd" || s == "ltccnh" || s == "ltccny")
      {
         return "https://www.okcoin.com/api/depth.do?symbol=ltc_usd&ok=1";
      }

      return "https://www.okcoin.com/api/depth.do?ok=1";
   }
   
   virtual string RequestUrl() const
   {
      return "https://www.okcoin.com/api";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int askStart = StringFind(s, "[[");
      int askEnd = StringFind(s, "]]", askStart);
      int bidStart = StringFind(s, "[[", askEnd);
      int bidEnd = StringFind(s, "]]", bidStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }
   
private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = StringSubstr(pair, 0, pos);
      if (StringGetChar(pStr, 0) == '\"') pStr = StringSubstr(pStr, 1, StringLen(pStr) - 2);
      string vStr = StringSubstr(pair, pos + 1);
      if (StringGetChar(vStr, 0) == '\"') vStr = StringSubstr(vStr, 1, StringLen(vStr) - 2);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class CoinbaseExchange
   : public IExchange
{
public:
   virtual string AdaptSymbol(string s) const
   {
      return s;
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = (depthLimit <= 50) ? "2" : "3";
      return "https://api.exchange.coinbase.com/products/btc-usd/book?level=" + s;
   }
   
   virtual string RequestUrl() const
   {
      return "https://api.exchange.coinbase.com";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      StringReplace(s, " ", "");
      
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int bidStart = StringFind(s, "[[");
      int bidEnd = StringFind(s, "]]", bidStart);
      int askStart = StringFind(s, "[[", bidEnd);
      int askEnd = StringFind(s, "]]", askStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }   

private:
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = StringSubstr(pair, 1, pos - 2);
      int pos2 = StringFind(pair, ",", pos + 1);

      if (pos2 == -1)
      {
         return (false);
      }

      string vStr = StringSubstr(pair, pos + 2, pos2 - pos - 2);
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

class PoloniexExchange
   : public IExchange
{
public:
   string Usdt(string s) const
   {
      if (s == "USD") return "USDT";
      return s;
   }
   
   virtual string AdaptSymbol(string s) const
   {
      string lower = ToUpper(s);
      return (StringConcatenate(Usdt(StringSubstr(lower, 3, 3)), "_", Usdt(StringSubstr(lower, 0, 3))));
   }

   virtual string GetUrl(string symbol, int depthLimit = -1) const
   {
      string s = StringFormat("https://poloniex.com/public?command=returnOrderBook&currencyPair=%s", symbol);
      if (depthLimit != -1) s = StringConcatenate(s, "&depth=" + IntegerToString(depthLimit));
      return s;
   }
   
   virtual string RequestUrl() const
   {
      return "https://poloniex.com/public";
   }
   
   virtual bool ParseResponse(string s, double& askP[], double& askV[], double& bidP[], double& bidV[])
   {
      ArrayResize(askP, 0);
      ArrayResize(askV, 0);
      ArrayResize(bidP, 0);
      ArrayResize(bidV, 0);
   
      int askStart = StringFind(s, "[[");
      int askEnd = StringFind(s, "]]", askStart);
      int bidStart = StringFind(s, "[[", askEnd);
      int bidEnd = StringFind(s, "]]", bidStart);
      
      if (askStart == -1 || askEnd == -1 || bidStart == -1 || bidEnd == -1)
      {
         return (false);
      }
      
      string asks = StringSubstr(s, askStart + 2, askEnd - askStart - 2);
      string bids = StringSubstr(s, bidStart + 2, bidEnd - bidStart - 2);
      
      // now parse pairs
      
      while (asks != "")
      {
         int pos = StringFind(asks, "],[");
         if (!Add(StringSubstr(asks, 0, pos), askP, askV)) return (false);
         
         if (pos == -1) asks = "";
         else           asks = StringSubstr(asks, pos + 3);
      }
   
      while (bids != "")
      {
         int pos = StringFind(bids, "],[");
         if (!Add(StringSubstr(bids, 0, pos), bidP, bidV)) return (false);
         
         if (pos == -1) bids = "";
         else           bids = StringSubstr(bids, pos + 3);
      }
      
      SortPrice(askP, askV);
      SortPrice(bidP, bidV, false);
      
      if (Level2Depth > 0)
      {
         if (ArraySize(askP) > Level2Depth)
         {
            ArrayResize(askP, Level2Depth);
            ArrayResize(askV, Level2Depth);
         }
         
         if (ArraySize(bidP) > Level2Depth)
         {
            ArrayResize(bidP, Level2Depth);
            ArrayResize(bidV, Level2Depth);
         }
      }

      return (true);
   }
   
private:
   string NoQuotes(string s) const
   {
      string res = StringSubstr(s, 0, 1);
      if (res == "\"")
      {
         res = StringSubstr(s, 1, StringLen(s) - 2);
         return res;
      }
      return s;
   }
   
   bool Add(string pair, double& ps[], double& vs[])
   {
      int pos = StringFind(pair, ",");
      if (pos == -1)
      {
         return (false);
      }
      
      string pStr = NoQuotes(StringSubstr(pair, 0, pos));
      string vStr = NoQuotes(StringSubstr(pair, pos + 1));
      
      if (pStr == "" || vStr == "")
      {
         return (false);
      }
   
      int size = ArraySize(ps);
      
      ArrayResize(ps, size + 1);
      ArrayResize(vs, size + 1);
      
      ps[size] = StrToDouble(pStr);
      vs[size] = StrToDouble(vStr);
      
      return (true);
   }
};

IExchange* CreateExchange(Exchanges e)
{
   IExchange* ret = NULL;
   switch (e)
   {
      case ExchangeBtce: ret = new BtceExchange; break;
      case ExchangeBitfinex: ret = new BitfinexExchange; break;
      case ExchangeBitstamp: ret = new BitstampExchange; break;
      case ExchangeHuobi: ret = new HuobiExchange; break;
      case ExchangeOkcoin: ret = new OkcoinExchange; break;
      case ExchangeCoinBase: ret = new CoinbaseExchange; break;
      case ExchangePoloniex: ret = new PoloniexExchange; break;
      default:
         Alert("Implementation error!");
         return NULL;
   }
   return ret;
}

double Pt;
double Dg;

datetime HDelta = 0;
datetime HStartDelta = 0;

bool HDeltaSet = false;
bool HStartDeltaSet = false;

void SetPoint(double pt)
{
   Pt = pt;
}

void SetDigits(double digits)
{
   Dg = digits;
}

void SortPrice(double& prices[], double& vols[], bool ascending = true)
{
   int size = ArraySize(prices);
   if (ArraySize(vols) != size)
   {
      Alert("logical error SortPrice -- prices != vols size");
      return;
   }
   
   double toSort[][2];
   ArrayResize(toSort, size);

   for (int i = 0; i < size; i++)
   {
      toSort[i][0] = prices[i];
      toSort[i][1] = vols[i];
   }
   
   int direction = MODE_DESCEND;
   if (ascending) direction = MODE_ASCEND;
   
   ArraySort(toSort, WHOLE_ARRAY, 0, direction);

   for (int i = 0; i < size; i++)
   {
      prices[i] = toSort[i][0];
      vols[i] = toSort[i][1];
   }
}

bool Merge(double& prices[], double& vols[], double& outPrices[], double& outVols[])
{
   double max = -10000000, min = 10000000;
   for (int i = 0; i < ArraySize(prices); i++)
   {
      if (prices[i] == 0) continue;
      if (max < prices[i]) max = prices[i];
      if (min > prices[i]) min = prices[i];
   }
   
   if (max < min) return (false);
   
   max = NormalizeDouble(MathRound((max + 0.5*Pt)/Pt)*Pt + 0.1*Pt, (int)Dg);
   min = NormalizeDouble(MathRound((min - 0.5*Pt)/Pt)*Pt - 0.1*Pt, (int)Dg);
   
   int size = (int)MathMax(MathRound((max - min)/Pt), 2);
   if (size == 0) return (false);

   double v[];
   ArrayResize(v, size);
   ArrayInitialize(v, 0);
   
   double d = max - min;
   for (int i = 0; i < ArraySize(prices); i++)
   {
      if (prices[i] == 0) continue;
      int idx = (int)MathRound((size - 1)*(prices[i] - min)/d);
      v[idx] += vols[i];
   }
   
   // transform to prices
   ArrayResize(outPrices, size);
   ArrayResize(outVols, size);
   
   for (int i = 0; i < size; i++)
   {
      outPrices[i] = min + i*(max - min)/(size);
      outVols[i] = v[i];
   }
   return (true);
}


void DrawRect(string name, datetime x1, datetime x2, double y1, double y2, color clr)
{
   ObjectCreate(name, OBJ_RECTANGLE, 0, x1, y1, x2, y2);
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_PRICE1, y1);
   ObjectSet(name, OBJPROP_PRICE2, y2);
   ObjectSet(name, OBJPROP_TIME1, x1);
   ObjectSet(name, OBJPROP_TIME2, x2);
}

void DrawLine(string name, datetime x1, datetime x2, double y1, double y2, color clr)
{
   ObjectCreate(name, OBJ_TREND, 0, x1, y1, x2, y2);
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_PRICE1, y1);
   ObjectSet(name, OBJPROP_PRICE2, y2);
   ObjectSet(name, OBJPROP_TIME1, x1);
   ObjectSet(name, OBJPROP_TIME2, x2);
   ObjectSet(name, OBJPROP_RAY, false);
   ObjectSet(name, OBJPROP_STYLE, STYLE_DASH);
   ObjectSet(name, OBJPROP_WIDTH, 1);
}

void DrawText(string name, datetime x1, double y1, string text, color clr)
{
   ObjectCreate(name, OBJ_TEXT, 0, x1, y1);
   ObjectSet(name, OBJPROP_COLOR, clr);
   ObjectSet(name, OBJPROP_PRICE1, y1);
   ObjectSet(name, OBJPROP_TIME1, x1);
   ObjectSetText(name, text, 10, "Arial Black");
}

void RemoveIrrelevant(string name, int pos)
{
   while(ObjectDelete(name + IntegerToString(pos)))
   {
      pos++;
   }
}

void DrawDepth(double& asks[], double& askVols[], double& bids[], double& bidVols[], datetime x, datetime dx, bool needScale, double k)
{
   color chartColor = (color)ChartGetInteger(0, CHART_COLOR_BACKGROUND);
   
   int askSize = ArraySize(asks);
   if (ArraySize(askVols) != askSize)
   {
      Alert("logical error DrawDepth -- askVols != asks size");
      return;
   }
   
   int bidSize = ArraySize(bids);
   if (ArraySize(bidVols) != bidSize)
   {
      Alert("logical error DrawDepth -- bidVols != bids size");
      return;
   }
   
   double sortAsks[], sortAskVols[], sortBids[], sortBidVols[];
   bool merged = true;
   merged = merged && Merge(asks, askVols, sortAsks, sortAskVols);
   merged = merged && Merge(bids, bidVols, sortBids, sortBidVols);
   
   if (!merged) 
   {
      return;
   }
   
   SortPrice(sortAsks, sortAskVols);
   SortPrice(sortBids, sortBidVols, false);
   
   askSize = ArraySize(sortAsks);
   bidSize = ArraySize(sortBids);

   double askVolume = 0;
   double bidVolume = 0;
   
   double cumAskVols[], cumBidVols[];
   ArrayResize(cumAskVols, askSize);
   ArrayResize(cumBidVols, bidSize);
   
   double max = 0;
   double cumMax = 0;
   
   for (int i = 0; i < askSize; i++)
   {
      askVolume += sortAskVols[i];
      cumAskVols[i] = askVolume;
      if (sortAskVols[i] > max) max = sortAskVols[i];
   }
   if (askVolume > cumMax) cumMax = askVolume;

   for (int i = 0; i < bidSize; i++)
   {
      bidVolume += sortBidVols[i];
      cumBidVols[i] = bidVolume;
      if (sortBidVols[i] > max) max = sortBidVols[i];
   }
   if (bidVolume > cumMax) cumMax = bidVolume;

   double rAskVols[], rBidVols[], rCumAskVols[], rCumBidVols[];
   ArrayResize(rAskVols, askSize);
   ArrayResize(rCumAskVols, askSize);
   ArrayResize(rBidVols, bidSize);
   ArrayResize(rCumBidVols, bidSize);
   
   double rMinAskVols[], rMinBidVols[];
   ArrayResize(rMinAskVols, askSize);
   ArrayResize(rMinBidVols, bidSize);

   for (int i = 0; i < askSize; i++)
   {
      rAskVols[i] = sortAskVols[i]/max;
      rCumAskVols[i] = cumAskVols[i]/cumMax;
      rMinAskVols[i] = MathMin(rAskVols[i], rCumAskVols[i]);
   }
   
   for (int i = 0; i < bidSize; i++)
   {
      rBidVols[i] = sortBidVols[i]/max;
      rCumBidVols[i] = cumBidVols[i]/cumMax;
      rMinBidVols[i] = MathMin(rBidVols[i], rCumBidVols[i]);
   }
   
   if (!HStartDeltaSet)
   {
      HStartDeltaSet = true;
      HStartDelta = 0;
   }
   
   if (!HDeltaSet)
   {
      HDeltaSet = true;
      HDelta = dx;
   }
   
   if (ObjectFind("1MarketDepthStartV") == 0)
   {
      if (ObjectFind("0MarketDepthStartV") == 0)
      {
         datetime x0 = (datetime)ObjectGet("0MarketDepthStartV", OBJPROP_TIME1);
         datetime x1 = (datetime)ObjectGet("1MarketDepthStartV", OBJPROP_TIME1);
         
         if (x0 != x1)
         {
            HStartDelta = x0 - x;
         }
      }
   }
   
   datetime xStart = x + HStartDelta;

   ObjectCreate("1MarketDepthStartV", OBJ_VLINE, 0, xStart, 0);
   ObjectSet("1MarketDepthStartV", OBJPROP_TIME1, xStart);
   ObjectSet("1MarketDepthStartV", OBJPROP_COLOR, CLR_NONE);
   ObjectSet("1MarketDepthStartV", OBJPROP_WIDTH, 1);

   ObjectCreate("0MarketDepthStartV", OBJ_VLINE, 0, xStart, 0);
   ObjectSet("0MarketDepthStartV", OBJPROP_TIME1, xStart);
   ObjectSet("0MarketDepthStartV", OBJPROP_COLOR, Red);
   ObjectSet("0MarketDepthStartV", OBJPROP_WIDTH, 1);

   if (ObjectFind("1MarketDepthSizeV") == 0)
   {
      if (ObjectFind("0MarketDepthSizeV") == 0)
      {
         datetime x0 = (datetime)ObjectGet("0MarketDepthSizeV", OBJPROP_TIME1);
         datetime x1 = (datetime)ObjectGet("1MarketDepthSizeV", OBJPROP_TIME1);
         
         if (x0 != x1)
         {
            HDelta = x0 - x;
         }
      }
   }
   
   datetime xEnd = x + HDelta;
   
   ObjectCreate("1MarketDepthSizeV", OBJ_VLINE, 0, xEnd, 0);
   ObjectSet("1MarketDepthSizeV", OBJPROP_TIME1, xEnd);
   ObjectSet("1MarketDepthSizeV", OBJPROP_COLOR, CLR_NONE);
   ObjectSet("1MarketDepthSizeV", OBJPROP_WIDTH, 1);

   ObjectCreate("0MarketDepthSizeV", OBJ_VLINE, 0, xEnd, 0);
   ObjectSet("0MarketDepthSizeV", OBJPROP_TIME1, xEnd);
   ObjectSet("0MarketDepthSizeV", OBJPROP_COLOR, Red);
   ObjectSet("0MarketDepthSizeV", OBJPROP_WIDTH, 1);
   
   DrawRect("0_BG", xStart, xEnd, sortBids[bidSize - 1] - Pt, sortAsks[askSize - 1] + Pt, Back);

   int i = 0;
   for (i = 0; i < askSize; i++)
   {
      DrawRect("1_cask"+ (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rCumAskVols[i]), sortAsks[i] - 0.5*Pt, sortAsks[i] + 0.5*Pt, Bulls ^ Back ^ chartColor);
      DrawRect("2_ask" + (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rMinAskVols[i]), sortAsks[i] - 0.4*Pt, sortAsks[i] + 0.4*Pt, Bulls ^ Back ^ BullLevels ^ InnerBullLevels ^ chartColor);
      DrawRect("3_ask" + (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rAskVols[i]), sortAsks[i] - 0.4*Pt, sortAsks[i] + 0.4*Pt, BullLevels ^ Back ^ chartColor);
   }
   
   i = askSize;
   while(ObjectFind("1_cask" + (string)i) != -1)
   {
      ObjectDelete("1_cask"+ (string)i);
      ObjectDelete("2_ask" + (string)i);
      ObjectDelete("3_ask" + (string)i);
      i++;
   }
   
   for (i = 0; i < bidSize; i++)
   {
      DrawRect("1_cbid"+ (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rCumBidVols[i]), sortBids[i] - 0.5*Pt, sortBids[i] + 0.5*Pt, Bears ^ Back ^ chartColor);
      DrawRect("2_bid" + (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rMinBidVols[i]), sortBids[i] - 0.4*Pt, sortBids[i] + 0.4*Pt, Bears ^ Back ^ BearLevels ^ InnerBearLevels ^ chartColor);
      DrawRect("3_bid" + (string)i, xStart, (datetime)(xStart + (xEnd - xStart)*rBidVols[i]), sortBids[i] - 0.4*Pt, sortBids[i] + 0.4*Pt, BearLevels ^ Back ^ chartColor);
   }

   i = bidSize;
   while(ObjectFind("1_cbid" + (string)i) != -1)
   {
      ObjectDelete("1_cbid"+ (string)i);
      ObjectDelete("2_bid" + (string)i);
      ObjectDelete("3_bid" + (string)i);
      i++;
   }
   
   //calculate scales
   double d = CalculateScale(max);
   double cumD = CalculateScale(cumMax);
   
   i = 0;
   double pos = d;
   while (pos < max)
   {
      DrawLine("scale_line_" + (string)i, (datetime)(xStart + (xEnd - xStart)*(pos/max)), (datetime)(xStart + (xEnd - xStart)*(pos/max)), sortBids[bidSize - 1] - Pt, sortAsks[askSize - 1] + Pt, ScaleColor);
      DrawText("text_for_scale" + (string)i, (datetime)(xStart + (xEnd - xStart)*(pos/max)), sortAsks[askSize - 1] + 0.1*(sortAsks[askSize - 1] - sortBids[bidSize - 1]), DoubleToStr(pos, ScaleDigits), ScaleColor);
      pos += d;
      i++;
   }
   DrawText("text_for_scale" + (string)i, (datetime)(xStart + 1.1*(xEnd - xStart)), sortAsks[askSize - 1] + 0.1*(sortAsks[askSize - 1] - sortBids[bidSize - 1]), DoubleToStr(max, ScaleDigits), ScaleColor);
   RemoveIrrelevant("scale_line_", i);
   RemoveIrrelevant("text_for_scale", i + 1);
   
   i = 0;
   pos = cumD;
   while (pos < cumMax)
   {
      DrawLine("cum_scale_line_" + (string)i, (datetime)(xStart + (xEnd - xStart)*(pos/cumMax)), (datetime)(xStart + (xEnd - xStart)*(pos/cumMax)), sortBids[bidSize - 1] - Pt, sortAsks[askSize - 1] + Pt, CumScaleColor);
      DrawText("text_for_cum_scale" + (string)i, (datetime)(xStart + (xEnd - xStart)*(pos/cumMax)), sortBids[bidSize - 1], DoubleToStr(pos, ScaleDigits), CumScaleColor);
      pos += cumD;
      i++;
   }
   DrawText("text_for_cum_scale" + (string)i, (datetime)(xStart + 1.1*(xEnd - xStart)), sortBids[bidSize - 1], DoubleToStr(cumMax, ScaleDigits), CumScaleColor);
   RemoveIrrelevant("cum_scale_line_", i);
   RemoveIrrelevant("text_for_cum_scale", i + 1);
   
   if (needScale)
   {
      DrawText("price_high", xStart, sortAsks[askSize - 1], DoubleToStr(sortAsks[askSize - 1]/k, Digits()) + "                ", PriceColor);
      double v = (sortAsks[0] + sortBids[0])/2;
      DrawText("price_mid", xStart, v, DoubleToStr(v/k, Digits()) + "                ", PriceColor);
      DrawText("price_low", xStart, sortBids[bidSize - 1], DoubleToStr(sortBids[bidSize - 1]/k, Digits()) + "                ", PriceColor);
   }
}

double CalculateRoundScale(double scale)
{
   double mul = 1;
   int norm = 0;
   while (scale > 10.0)
   {
      mul *= 10;
      norm--;
      scale /= 10.0;
   }
   while (scale < 1.0)
   {
      mul /= 10;
      norm++;
      scale *= 10.0;
   }
   
   double rnd = NormalizeDouble(NormalizeDouble(scale, 1)*mul, MathMax(norm + 1, 0));
   if (scale > 3) rnd = NormalizeDouble(NormalizeDouble(scale, 0)*mul, MathMax(norm, 0));
   return rnd;
}

double CalculateScale(double max)
{
   int mul = 1;
   while (max > 10)
   {
      mul *= 10;
      max /= 10;
   }
   
   if (max > 9) return (3.5*mul);
   if (max > 7.5) return (3*mul);
   if (max > 6) return (2.5*mul);
   if (max > 4.5) return (2*mul);
   if (max > 3.5) return (1.5*mul);
   if (max > 2.5) return (2*mul);
   if (max > 1.8) return (1.5*mul);
   if (max > 1.2) return (mul);
   return (0.5*mul);
}

IExchange* Ex = NULL;

bool ReceiveData(IExchange* e, string symbol, string& response, int timeout = -1)
{
   string headers;
   char post[], result[];
   if (timeout == -1) timeout = 10000;
   
   int res = WebRequest("GET", e.GetUrl(symbol, Level2Depth), NULL, NULL, timeout, post, 0, result, headers);
   if(res == -1)
   {
      int err = GetLastError();
      Print("WebRequest error #", err);
      static bool alerted = false;
      if (err == ERR_FUNCTION_NOT_CONFIRMED && !alerted)
      {
         Alert("Please allow Web Requests! for URL \"" + e.RequestUrl() + "\"");
         alerted = true;
      }
      return false;
   }

   response = CharArrayToString(result);
   return true;
}

void Multiply(double& a[], double k)
{
   int size = ArraySize(a);
   for (int i = 0; i < size; i++)
   {
      a[i] *= k;
   }
}

int start()
{
   Ex = CreateExchange(Exchange);
   HDelta = 0;
   HStartDelta = 0;

   HDeltaSet = false;
   HStartDeltaSet = false;

   datetime t = 0;
   if (!AutoPointSize)
   {
      SetPoint(PointSize);
   }
   else
   {
      //assume that convenient separable scale is 0.1% from the chart price
      SetPoint(CalculateRoundScale((Ask + Bid)*0.001/2.0));
   }
   SetDigits(Digits);
   
   while (!IsStopped())
   {
      datetime current = TimeLocal();
      if (current - t > RefreshSeconds)
      {
         t = current;
         
         string symbol = Ex.AdaptSymbol(Symbol());
         if (CustomInstrument != "")
         {
            symbol = CustomInstrument;
         }
         
         string response = "";
         if (ReceiveData(Ex, symbol, response))
         {
            double askP[], askV[], bidP[], bidV[];
            bool res = Ex.ParseResponse(response, askP, askV, bidP, bidV);
            
            if (!res)
            {
               Print("Parse Failed");
            }
            else
            {
               bool needScale = false;
               double k = 1;
               if (AutoScale)
               {
                  needScale = true;
                  k = (Ask + Bid)/(askP[0] + bidP[0]);
                  Multiply(askP, k);
                  Multiply(bidP, k);
               }
               else if (CustomScale != 1.0)
               {
                  needScale = true;
                  k = CustomScale;
                  Multiply(askP, CustomScale);
                  Multiply(bidP, CustomScale);
               }
            
               RefreshRates();
               DrawDepth(askP, askV, bidP, bidV, Time[0], 9000*Period(), needScale, k);
               WindowRedraw();
            }
         }
      }
      Sleep(2000);
   }
   
   ObjectDelete("0_BG");
   ObjectDelete("0MarketDepthSizeV");
   ObjectDelete("1MarketDepthSizeV");
   ObjectDelete("0MarketDepthStartV");
   ObjectDelete("1MarketDepthStartV");
   
   
   for (int i = ObjectsTotal(); i >= 0; i--)
   {
      string name = ObjectName(i);

#define DELETE_BY_NAME(x) if (StringFind(name, x) != -1) {ObjectDelete(name);continue;}

      DELETE_BY_NAME("1_cask");
      DELETE_BY_NAME("2_ask");
      DELETE_BY_NAME("3_ask");
      DELETE_BY_NAME("1_cbid");
      DELETE_BY_NAME("2_bid");
      DELETE_BY_NAME("3_bid");

      DELETE_BY_NAME("scale_line_");
      DELETE_BY_NAME("cum_scale_line_");
      DELETE_BY_NAME("text_for_scale");
      DELETE_BY_NAME("text_for_cum_scale");
      
      DELETE_BY_NAME("price_high");
      DELETE_BY_NAME("price_mid");
      DELETE_BY_NAME("price_low");
   }
   
   delete Ex;
   
   return(0);
}

string ToLower(string s)
{
   string low = s;
   StringToLower(low);
   return low;
}

string ToUpper(string s)
{
   string up = s;
   StringToUpper(up);
   return up;
}